
      LOGICAL FUNCTION H2XML( hdir, hid, directory, filenumber, colour )
c     ------------------------------------------------------------------
c     Function to write histogram in memory at hpath to file called
c     plot{filenumber}.xml in directory {directory}.  
c
c     The output format is JAS plotML
c     (see http://jas.freehep.org/)
c     Jon Butterworth, 16/6/2001
c
c     hdir = character*160, hbook directory
c     hid = integer, histogram ID
c     directory = character*160 full path to destination directory of
c     xml output
c     filenumber = xml file will be called ploti.xml i=filenumber
c     colour = colour in which plot will be drawn
c
c     ------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER i, lenocc

      CHARACTER*160 hdir, filename, directory
      INTEGER hid, filenumber

c     Histogram information.
      CHARACTER*160 title, colour
      
      REAL xmi,xma,ymi,yma,x,x2
      
      INTEGER loc, nwt
      INTEGER nx, ny, nx1, ny1, ierr, ndat

      INTEGER kind(50)

      INTEGER maxbin
      PARAMETER(maxbin = 200)

      INTEGER unit
      PARAMETER(unit = 13)

      REAL errors(maxbin), dpoint(maxbin)

      CHARACTER*2 num

      filename = directory(1:lenocc(directory))//'/plot'
      IF (filenumber.LT.10) THEN
         num = CHAR(filenumber+48)
      ELSE IF (filenumber.LT.20) THEN
         num = CHAR(49)//CHAR(filenumber-10+48)
      ELSE IF (filenumber.LT.30) THEN
         num = CHAR(50)//CHAR(filenumber-20+48)
      ELSE IF (filenumber.LT.40) THEN
         num = CHAR(51)//CHAR(filenumber-30+48)
      ELSE IF (filenumber.LT.50) THEN
         num = CHAR(52)//CHAR(filenumber-40+48)
      ELSE IF (filenumber.LT.60) THEN
         num = CHAR(53)//CHAR(filenumber-50+48)
      ELSE IF (filenumber.LT.70) THEN
         num = CHAR(54)//CHAR(filenumber-60+48)
      ELSE IF (filenumber.LT.80) THEN
         num = CHAR(55)//CHAR(filenumber-70+48)
      ELSE IF (filenumber.LT.90) THEN
         num = CHAR(56)//CHAR(filenumber-80+48)
      ELSE
         num = CHAR(57)//CHAR(filenumber-90+48)
      ENDIF

      filename = filename(1:lenocc(filename))//num(1:lenocc(num))/
     &     /'.xml'
      write(*,*) 'filename:',filename

      CALL HCDIR(hdir(1:LENOCC(hdir)),' ')

      h2xml = .FALSE.

c     Get information about this histogram.
      CALL HGIVE(hid,title,nx,xmi,xma,ny,ymi,yma,nwt,loc)

c     Check the number of bins is ok.
      IF (nx.GT.maxbin) THEN
         WRITE(*,*) 'Too many bins: ',nx,title 
         RETURN
      ENDIF

c     
      CALL HUNPAK(hid,dpoint,' ',1)
      CALL HUNPKE(hid,errors,' ',1)

c     Open the file.
      OPEN(UNIT=unit,FILE=filename(1:lenocc(filename)),STATUS='NEW')

c     Write out the header stuff.
      WRITE(unit,1000) 
c      WRITE(unit,1001) 
      WRITE(unit,1002) 
      WRITE(unit,1003) 
c      WRITE(unit,1004) title 
      WRITE(unit,1004) "  " 
      WRITE(unit,1005) 
      WRITE(unit,1006) 
      WRITE(unit,1007) colour(1:LENOCC(colour))
      WRITE(unit,1008) 
      WRITE(unit,1009) 
      WRITE(unit,1010) 

      CALL HKIND(hid,kind,'A')
      IF (kind(6).EQ.0) THEN
c     Easy case. Fixed bin widths
      
      WRITE(unit,2001) 
      DO i=1,nx
         
         IF (dpoint(i).GE.0) THEN
            WRITE(unit,5000) dpoint(i),errors(i) 
         ELSE
            WRITE(unit,5000) 0.,0.
         ENDIF
         
      ENDDO
      WRITE(unit,2003) 
      WRITE(unit,2004) xmi,xma
      IF (nx.LT.10) THEN
         WRITE(unit,2005) nx 
      ELSE
         WRITE(unit,2015) nx 
      ENDIF
      WRITE(unit,2006) 
      WRITE(unit,2007) 
      WRITE(unit,2008) 
      WRITE(unit,2009) 
      WRITE(unit,2010) 
      WRITE(unit,2011) 
      WRITE(unit,2012) 

      ELSE

c     Tricky. variable bin widths
         DO i=1,nx
            CALL HIX(hid,i,x)
            IF (i.LT.nx) THEN
               CALL HIX(hid,i+1,x2)
            ELSE
               x2=xma
            ENDIF
            WRITE(unit,2001) 
            IF (dpoint(i).GE.0) THEN 
               WRITE(unit,5000) dpoint(i),errors(i) 
            ELSE
               WRITE(unit,5000) 0.,0.
            ENDIF
            WRITE(unit,2003) 
            WRITE(unit,2004) x,x2
            WRITE(unit,2005) 1 
            WRITE(unit,2006) 
            WRITE(unit,2007) 
            WRITE(unit,2008) 
            WRITE(unit,2009) 
            WRITE(unit,2010) 
            WRITE(unit,2011) 
            WRITE(unit,2012) 
         ENDDO
         
      ENDIF

c     Now finishing off.
      WRITE(unit,1020) 
      WRITE(unit,1021) 
      WRITE(unit,1022) 
      WRITE(unit,1023) 
      WRITE(unit,1024) 
      WRITE(unit,1025) 
      WRITE(unit,1026) 
      WRITE(unit,1027) 

      WRITE(unit,1030) 
      WRITE(unit,1031) 
      WRITE(unit,1032) 
      WRITE(unit,1033) 
      WRITE(unit,1034) 
      WRITE(unit,1035) 
      WRITE(unit,1036) 
      WRITE(unit,1037) 

      WRITE(unit,1040) 
      WRITE(unit,1041) 
      WRITE(unit,1042) 
      WRITE(unit,1043) 


      CLOSE(13)

      RETURN 

 1000 FORMAT('<?xml version="1.0" encoding="ISO-8859-1" ?>')
 1001 FORMAT('<!DOCTYPE plotML SYSTEM "plotML.dtd">')
 1002 FORMAT('<plotML>',/,'<plot>')
 1003 FORMAT('<title>',/,'<border type="None"/>')
 1004 FORMAT('<label text="',A,'">',/
     &     ,'<font style="BOLD" points="14" face="SansSerif"/>'
     &     ,'</label>')
 1005 FORMAT('</title>')
 1006 FORMAT('<legend visible="never">')
 1007 FORMAT('<border color="',A,'" type="Line"/>')
 1008 FORMAT('<font style="PLAIN" points="12" face="Dialog"/>')
 1009 FORMAT('</legend>')
 1010 FORMAT('<dataArea>')


 1020 FORMAT('<axis logarithmic="false" showOverflows="false"')
 1021 FORMAT('numberOfBins="40" allowSuppressedZero="true"')
 1022 FORMAT('type="double" location="x0">')
 1023 FORMAT('<label text="ET (GeV)">')
 1024 FORMAT('<font style="PLAIN" face="Dialog" points="12"/>')
 1025 FORMAT('</label>')
 1026 FORMAT('<font style="PLAIN" face="Dialog" points="12"/>')
 1027 FORMAT('</axis>')
 1030 FORMAT('<axis logarithmic="false" showOverflows="false"')
 1031 FORMAT('numberOfBins="40" allowSuppressedZero="true"')
 1032 FORMAT('type="double" location="y0">')
 1033 FORMAT('<label text="Cross section (nb)">')
 1034 FORMAT('<font style="PLAIN" face="Dialog" points="12"/>')
 1035 FORMAT('</label>')
 1036 FORMAT('<font style="PLAIN" face="Dialog" points="12"/>')
 1037 FORMAT('</axis>')

 1040 FORMAT('</dataArea>')
 1041 FORMAT('<bounds width="400" height="400"/>')
 1042 FORMAT('<colorScheme backgroundColor="(255,215,0)"/>')
 1043 FORMAT('</plot>',/,'</plotML>')

 2001 FORMAT('<data1d>',/,'<bins1d title=" ">')
 2002 FORMAT(F8.4,',',F8.4)
 2003 FORMAT('</bins1d>')
 2004 FORMAT('<binnedDataAxisAttributes min="',F8.2,'" max="',F8.2,'"')
 2005 FORMAT('numberOfBins="',I1,'" axis="x0" type="double"/>')
 2015 FORMAT('numberOfBins="',I2,'" axis="x0" type="double"/>')
 2006 FORMAT('<style1d histogramBarsFilled="true"')
 2007 FORMAT('histogramBarColor="Green"')
 2008 FORMAT('errorBarColor="Black" dataPointColor="Black"') 
 2009 FORMAT('dataPointStyle="dot" dataPointSize="6"') 
 2010 FORMAT('showHistogramBars="false" showErrorBars="true"')
 2011 FORMAT('showDataPoints="true"/>')
 2012 FORMAT('</data1d>')

 5000 FORMAT(E8.3,',',E8.3)

      END

     


