      PROGRAM PYT
C
C     Pythia main program for HzTool/JetWeb.
C
C     N.B. Use only for unweighted running!
C
      IMPLICIT NONE

c     HBOOK stuff
      INTEGER NWPAWC,H
      PARAMETER (NWPAWC=9000000)
      COMMON/PAWC/H(NWPAWC)

      EXTERNAL PYDATA

c     Counter used in event loop
      INTEGER EVLOOP
      INTEGER ILOOP

c     Total cross section [pb]
      REAL XSECPB

c     Mean charged multiplicity
      DOUBLE PRECISION nchavg

c     Temporary variables for output of HZNEVT
      INTEGER HZNEV2
      REAL HZLUMI2

c     Beam particles
      CHARACTER*8 part1, part2

c     Event type: 1 = pbar-p, 2 = ep, 3 = ee
c      INTEGER ITYPLOC

c     Offset for IFLAG
      INTEGER IFLOFF

c     Process loop
      INTEGER JPROC

c     HEPEVT prime common block
#include "hepevtp.inc"

c     Misleadingly named common block
c     Contains information on generator, no. of events etc.
#include "heracmn.inc"

c     Pythia common blocks
#include "hzpytsteer.inc"

c     Common block with steering parameters
#include "hzcontrol.inc"
#include "hzpytcontrol.inc"

c     Move to common block, e.g. HZFUNC?
      DOUBLE PRECISION HZPYAVRG

      PRINT*,''
      PRINT*,'*******************************************************'
      PRINT*,'*'
      PRINT*,'* jetweb_pyt.fpp $Date: 2005/05/26 15:07:37 $'  
      PRINT*,'*'
      PRINT*,'*******************************************************'
      PRINT*,''

      GEN = 'PYT'

c     Read steering cards etc.
      CALL HZCARDS(1)
      CALL HZPYTCARDS
      CALL HZCARDS(2)

c     Create local copy of beam particle types (and convert to upper case using KERNLIB util)
c     Don't really need to convert to upper case but makes reusing HERWIG code easier
      PART1 = hzb1
      PART2 = hzb2
      CALL CLTOU(PART1)
      CALL CLTOU(PART2)

c     Set beam momenta: 1st beam has positive pz
      P(1,1)=0.
      P(1,2)=0.
      P(1,3)=hzpb1
      P(2,1)=0.
      P(2,2)=0.
      P(2,3)=-hzpb2

c     Begin loop over the MSTP14 options.
      DO jproc=1,hzmaxmstp14

         IF (hzmstp14(jproc).EQ.-999.and.jproc.EQ.1) THEN
c     Special case - only one run, mstp14 not set.
            ifloff = 0
         ELSE IF (hzmstp14(jproc).NE.-999) THEN
c     Offset according to process number.
            ifloff = 1000*jproc
            mstp(14) = hzmstp14(jproc)
         ELSE
            goto 999
         ENDIF

c     Set subprocesses
         msel = hzmsel
         DO 21 iloop = 1, 500
            msub(iloop) = hzmsub(iloop)
 21      CONTINUE

C     Copy MDME1 from steering cards
       DO 211 iloop = 1, 4285
         IF (HZMDME1(ILOOP).GT.-9999) MDME(ILOOP,1) = HZMDME1(ILOOP)
  211   CONTINUE
         
c     Set minimum PT for hard subprocess
         ckin(3) = hzptmin
         
c     Copy MSTP, PARP, MSTJ, PARJ and CKIN from steering cards
         DO 22 iloop = 1, 200
            IF (HZPARP(ILOOP).GT.-9999.) PARP(ILOOP) = HZPARP(ILOOP)
            IF (HZMSTP(ILOOP).GT.-9999) MSTP(ILOOP) = HZMSTP(ILOOP)
            IF (HZPARJ(ILOOP).GT.-9999.) PARJ(ILOOP) = HZPARJ(ILOOP)
            IF (HZMSTJ(ILOOP).GT.-9999) MSTJ(ILOOP) = HZMSTJ(ILOOP)
            IF (HZCKIN(ILOOP).GT.-9999.) CKIN(ILOOP) = HZCKIN(ILOOP)
 22      CONTINUE
         
c     Set PDFs of beam particles using PDFLIB, overriding MSTP
         IF (hzpdfpr.GT.-9999) THEN
            MSTP(51) = hzpdfpr
            MSTP(52) = 2
         ENDIF
         IF (hzpdfph.GT.-9999) THEN
            MSTP(55) = hzpdfph
            MSTP(56) = 2
         ENDIF
         
c     Set initial seed for random number generator
         mrpy(1)=hzrnd1
         WRITE(*,*) 'Using random number sequence ',mrpy(1)
         
c     Initialize Pythia for p/pbar collisions with beam 3-momenta
c     given by array P. Last argument is dummy.
         CALL PYINIT('3MOM',part1,part2,0.D0)
         
c     Need to know event type and whether direct or singly/double resolved so can add
c     offset to iflag.
c     MSTP(14) determines direct/resolved but not all possible values are dealt with here ???
c     Only allow for values used in HzTool v2-003 : 0, 1 (ep) and 21, 4, 24 (ee)
c         if ((PART1.eq.'PBAR'.and.PART2.eq.'P').or. (PART2.eq.'PBAR'.and
c     $        .PART1.eq.'P')) then
c     ityploc = 1
c            write (*,*) '*** jetweb_pyt: pbar-p running'
c         else if (((PART1.eq.'E+'.or.PART1.eq.'E-').and.PART2.eq.'P').or
c     $           .((PART2.eq.'E+'.or.PART2.eq.'E-').and.PART1.eq.'P'))
c     $           then
c     ityploc = 2
c            write (*,*) '*** jetweb_pyt: ep running'
c         else if ((PART1.eq.'E+'.and.PART2.eq.'E-').or. (PART2.eq.'E+'
c     $           .and.PART1.eq.'E-')) then
c     ityploc = 3
c            write (*,*) '*** jetweb_pyt: e+e- running'
c         else
c         ityploc = 0
c            WRITE(*,*)
c     $           '*** Warning in jetweb_pyt: unknown collision type'
c         end if
c     ifloff = 0
c      if (ityploc.eq.2) then
c     ep direct
c         if (MSTP(14).eq.0) ifloff = 1000
c     ep resolved
c         if (MSTP(14).eq.1) ifloff = 2000
c      else if (ityploc.eq.3) then
c     ee direct
c         if (MSTP(14).eq.21) ifloff = 1000
c     ee singly resolved
c         if (MSTP(14).eq.4) ifloff = 2000
c     ee doubly resolved
c         if (MSTP(14).eq.24) ifloff = 3000
c      end if
         

         write (*,*) '*** jetweb_pyt: iflag offset = ',ifloff

c     Switch off decays of final-state particles selected to be stable
         DO 23 iloop = 1, 200
            IF (HZSTAB(iloop).ne.-9999) THEN
               WRITE(*,*) 'Making particle stable: ',HZSTAB(iloop)
               MDCY(PYCOMP(HZSTAB(iloop)),1) = 0
            ENDIF
 23      CONTINUE
         
c     Generate a few events to estimate cross section
         write (6,*) 'Generating a few events to estimate cross section'
         DO EVLOOP=1,100
            CALL PYEVNT
         ENDDO
         CALL PYSTAT(1)

c     Get total cross section from Pythia in millibarns. Convert to pb.
         XSECPB = oldXSEC(0,3)*1E+09

c     Decide how many events to generate and corresponding luminosity
         CALL HZNEVT(XSECPB, HZNEV, HZLUMI, HZNEV2, HZLUMI2)
         HZNEV = HZNEV2

c     Calculate mean charged multiplicity, needed by some analysis routines

         write (hzunit,*)
     &      'Generating a few events to estimate mean multiplicity'
         nchavg = HZPYAVRG(5000)

c     Re-initialize Pythia to clear lumi and number of events etc.
         CALL PYINIT('3MOM',part1,part2,0.D0)

c     Switch off decays of final-state particles selected to be stable
         DO 24 iloop = 1, 200
            IF (HZSTAB(iloop).ne.-9999) THEN
               WRITE(*,*) 'Making particle stable: ',HZSTAB(iloop)
               MDCY(PYCOMP(HZSTAB(iloop)),1) = 0
            ENDIF
 24      CONTINUE

c     Write our some numbers to the log file:-
c     User-Defined Limits on Kinematical Variables
         CALL PYSTAT(4)
c     Summary of Status Codes and Parameter Values (common /PYPARS/)
         CALL PYSTAT(5)
c     List parameter value table (common /PYDAT1/, /PYDAT2/)
         CALL PYLIST(13)

c     Initialize HBOOK histogramming
         CALL HZHISTO(1+ifloff,nchavg)

c     Start XML file (only for the first process)
         IF (hzxmlout.ne.0.AND.jproc.EQ.1) THEN
c               CALL HZPYTXML(0)
c               CALL HZPYTXML(1)
c               CALL HZPYTXML(2)
c               CALL HZPYTXML(3)
         ENDIF

c     Generate events
         write (6,*) 'Starting main event generation loop'
         DO EVLOOP=1,HZNEV
c     Call Pythia to generate event
            CALL PYEVNT
c     Transfer information to HEPEVT common, HERACMN etc.
            CALL HZFILHEP
c     Write out first couple of events
            IF (EVLOOP.LE.2) CALL PYLIST(1)
c     Set event weight WTX as some analysis routines seem to use this
c     Only dealing with unweighted events at the moment
            WTX = 1.0
c     Fill histograms
            CALL HZHISTO(2+ifloff,nchavg)
         ENDDO
         
c     Print out some info from Pythia
         CALL PYSTAT(1)   
         
c     Fill number of events and cross section [nb] in HERACMN
         ntot=hznev
         xsec=oldxsec(0,3)*1E+06
         write(*,*) 'jetweb_pyt: ntot, xsec [nb] = ',ntot,xsec
         
c     Finish off HBOOK stuff
         CALL HZHISTO(3+ifloff,nchavg)

 999     CONTINUE
      ENDDO

c     Output histograms and close file.
      CALL HZHISTO(-1,nchavg)
      
c     Write out parameters as XML
c     At end of run so that luminosity, cross section etc. should all be known

      IF (hzxmlout.ne.0) THEN

c        CALL HZPYTXML(4)
c        CALL HZPYTXML(-3)

c     Finish and close XML file
c        CALL HZPYTXML(-1)

      ENDIF
      
      PRINT*,''
      PRINT*
     $     ,'*******************************************************'
      PRINT*,'*'
      PRINT*,'* Outputted and ended'    
      PRINT*,'*'
      PRINT*
     $     ,'*******************************************************'
      PRINT*,''

      END
