      SUBROUTINE HZNEVT(xsec, nevin, lumiin, nevout, lumiout) 
c
c     Decides how many events to generate, using whichever is smaller out of
c     (a) the number of events requested in the steering and
c     (b) the number calculated from the requested luminosity and the estimated
c         total cross section.
c
c     Try to be careful to deal sensibly with overflows, e.g. if luminosity
c     requested results in negative number of events.
c
c       Input:  xsec   = total cross section
c               nevin  = no. of events from steering
c               lumiin = luminosity from steering
c       Output: nevout  = no. of events to generate
c               lumiout = corresponding luminosity
c
      IMPLICIT NONE

      REAL xsec, lumiin, lumiout
      INTEGER nevin, nevout
      INTEGER nevlumi

      nevout = nevin

      nevlumi = xsec * lumiin

      WRITE (6,*)
     &     '***********************************************************'
      WRITE (6,*) '* Requested luminosity [pb^-1] = ',lumiin
      WRITE (6,*) '* Max allowed number of events = ',nevin
      WRITE (6,*) '* Number of events calculated from lumi = ',nevlumi

      IF (lumiin.lt.0.) THEN
        WRITE (6,*)
     &  '* Using requested number of events since lumi<0'
        nevout = nevin
        ELSE

          IF (nevlumi.LT.0) THEN
            WRITE (6,*) '* Warning: '//
     &        'overflow in number of events calculated from lumi'
          ELSEIF (nevlumi.GT.nevin) THEN
            WRITE (6,*)
     &        '* Number of events calculated from lumi greater than ',
     &        nevin
          ELSE
            nevout = nevlumi
          ENDIF
        ENDIF
      lumiout = nevout / xsec

      WRITE (6,*) '* Number of events to generate: ',nevout
c     IMPORTANT! The following line is used by JetWeb to read the luminosity. Do not change!
      WRITE (6,*) '* Lumi to generate (pb-1):      ',lumiout
      WRITE (6,*)
     &     '***********************************************************'

      END
